@@ -1,8 +1,8 @@
 /**********************************************************************
-  iso8859_2.c -  Oniguruma (regular expression library)
+  iso8859_15.c -  Oniguruma (regular expression library)
 **********************************************************************/
 /*-
- * Copyright (c) 2002-2006  K.Kosako  <sndgk393 AT ybb DOT ne DOT jp>
+ * Copyright (c) 2002-2007  K.Kosako  <sndgk393 AT ybb DOT ne DOT jp>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -29,11 +29,11 @@
 
 #include "regenc.h"
 
-#define ENC_ISO_8859_2_TO_LOWER_CASE(c) EncISO_8859_2_ToLowerCaseTable[c]
-#define ENC_IS_ISO_8859_2_CTYPE(code,ctype) \
-  ((EncISO_8859_2_CtypeTable[code] & CTYPE_TO_BIT(ctype)) != 0)
+#define ENC_ISO_8859_15_TO_LOWER_CASE(c) EncISO_8859_15_ToLowerCaseTable[c]
+#define ENC_IS_ISO_8859_15_CTYPE(code,ctype) \
+  ((EncISO_8859_15_CtypeTable[code] & CTYPE_TO_BIT(ctype)) != 0)
 
-static const UChar EncISO_8859_2_ToLowerCaseTable[256] = {
+static const UChar EncISO_8859_15_ToLowerCaseTable[256] = {
   '\000', '\001', '\002', '\003', '\004', '\005', '\006', '\007',
   '\010', '\011', '\012', '\013', '\014', '\015', '\016', '\017',
   '\020', '\021', '\022', '\023', '\024', '\025', '\026', '\027',
@@ -54,10 +54,10 @@ static const UChar EncISO_8859_2_ToLowerCaseTable[256] = {
   '\210', '\211', '\212', '\213', '\214', '\215', '\216', '\217',
   '\220', '\221', '\222', '\223', '\224', '\225', '\226', '\227',
   '\230', '\231', '\232', '\233', '\234', '\235', '\236', '\237',
-  '\240', '\261', '\242', '\263', '\244', '\265', '\266', '\247',
-  '\250', '\271', '\272', '\273', '\274', '\255', '\276', '\277',
-  '\260', '\261', '\262', '\263', '\264', '\265', '\266', '\267',
-  '\270', '\271', '\272', '\273', '\274', '\275', '\276', '\277',
+  '\240', '\241', '\242', '\243', '\244', '\245', '\250', '\247',
+  '\250', '\251', '\252', '\253', '\254', '\255', '\256', '\257',
+  '\260', '\261', '\262', '\263', '\270', '\265', '\266', '\267',
+  '\270', '\271', '\272', '\273', '\275', '\275', '\377', '\277',
   '\340', '\341', '\342', '\343', '\344', '\345', '\346', '\347',
   '\350', '\351', '\352', '\353', '\354', '\355', '\356', '\357',
   '\360', '\361', '\362', '\363', '\364', '\365', '\366', '\327',
@@ -68,7 +68,7 @@ static const UChar EncISO_8859_2_ToLowerCaseTable[256] = {
   '\370', '\371', '\372', '\373', '\374', '\375', '\376', '\377'
 };
 
-static const unsigned short EncISO_8859_2_CtypeTable[256] = {
+static const unsigned short EncISO_8859_15_CtypeTable[256] = {
   0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008,
   0x4008, 0x420c, 0x4209, 0x4208, 0x4208, 0x4208, 0x4008, 0x4008,
   0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008, 0x4008,
@@ -89,10 +89,10 @@ static const unsigned short EncISO_8859_2_CtypeTable[256] = {
   0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008,
   0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008,
   0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008, 0x0008,
-  0x0284, 0x34a2, 0x00a0, 0x34a2, 0x00a0, 0x34a2, 0x34a2, 0x00a0,
-  0x00a0, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x01a0, 0x34a2, 0x34a2,
-  0x00a0, 0x30e2, 0x00a0, 0x30e2, 0x00a0, 0x30e2, 0x30e2, 0x00a0,
-  0x00a0, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x00a0, 0x30e2, 0x30e2,
+  0x0284, 0x01a0, 0x00a0, 0x00a0, 0x00a0, 0x00a0, 0x34a2, 0x00a0,
+  0x30e2, 0x00a0, 0x30e2, 0x01a0, 0x00a0, 0x01a0, 0x00a0, 0x00a0,
+  0x00a0, 0x00a0, 0x10a0, 0x10a0, 0x34a2, 0x30e2, 0x00a0, 0x01a0,
+  0x30e2, 0x10a0, 0x30e2, 0x01a0, 0x34a2, 0x30e2, 0x34a2, 0x01a0,
   0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2,
   0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2,
   0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x34a2, 0x00a0,
@@ -100,12 +100,12 @@ static const unsigned short EncISO_8859_2_CtypeTable[256] = {
   0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2,
   0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2,
   0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x00a0,
-  0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x00a0
+  0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2, 0x30e2
 };
 
 static int
-iso_8859_2_mbc_case_fold(OnigCaseFoldType flag,
-                         const UChar** pp, const UChar* end, UChar* lower)
+mbc_case_fold(OnigCaseFoldType flag,
+	      const UChar** pp, const UChar* end ARG_UNUSED, UChar* lower)
 {
   const UChar* p = *pp;
 
@@ -116,15 +116,15 @@ iso_8859_2_mbc_case_fold(OnigCaseFoldType flag,
     return 2;
   }
 
-  *lower = ENC_ISO_8859_2_TO_LOWER_CASE(*p);
+  *lower = ENC_ISO_8859_15_TO_LOWER_CASE(*p);
   (*pp)++;
   return 1; /* return byte length of converted char to lower */
 }
 
 #if 0
 static int
-iso_8859_2_is_mbc_ambiguous(OnigCaseFoldType flag,
-			    const UChar** pp, const UChar* end)
+is_mbc_ambiguous(OnigCaseFoldType flag,
+		 const UChar** pp, const UChar* end)
 {
   int v;
   const UChar* p = *pp;
@@ -135,71 +135,79 @@ iso_8859_2_is_mbc_ambiguous(OnigCaseFoldType flag,
   }
 
   (*pp)++;
-  v = (EncISO_8859_2_CtypeTable[*p] & (BIT_CTYPE_UPPER | BIT_CTYPE_LOWER));
+  v = (EncISO_8859_15_CtypeTable[*p] & (BIT_CTYPE_UPPER | BIT_CTYPE_LOWER));
   if ((v | BIT_CTYPE_LOWER) != 0) {
-    return TRUE;
+    /* 0xdf etc.. are lower case letter, but can't convert. */
+    if (*p == 0xaa || *p == 0xb5 || *p == 0xba)
+      return FALSE;
+    else
+      return TRUE;
   }
 
   return (v != 0 ? TRUE : FALSE);
 }
 #endif
 
+static int
+is_code_ctype(OnigCodePoint code, unsigned int ctype)
+{
+  if (code < 256)
+    return ENC_IS_ISO_8859_15_CTYPE(code, ctype);
+  else
+    return FALSE;
+}
+
 static const OnigPairCaseFoldCodes CaseFoldMap[] = {
-  { 0xa1, 0xb1 },
-  { 0xa3, 0xb3 },
-  { 0xa5, 0xb5 },
-  { 0xa6, 0xb6 },
-  { 0xa9, 0xb9 },
-  { 0xaa, 0xba },
-  { 0xab, 0xbb },
-  { 0xac, 0xbc },
-  { 0xae, 0xbe },
-  { 0xaf, 0xbf },
+ { 0xa6, 0xa8 },
 
-  { 0xc0, 0xe0 },
-  { 0xc1, 0xe1 },
-  { 0xc2, 0xe2 },
-  { 0xc3, 0xe3 },
-  { 0xc4, 0xe4 },
-  { 0xc5, 0xe5 },
-  { 0xc6, 0xe6 },
-  { 0xc7, 0xe7 },
-  { 0xc8, 0xe8 },
-  { 0xc9, 0xe9 },
-  { 0xca, 0xea },
-  { 0xcb, 0xeb },
-  { 0xcc, 0xec },
-  { 0xcd, 0xed },
-  { 0xce, 0xee },
-  { 0xcf, 0xef },
+ { 0xb4, 0xb8 },
+ { 0xbc, 0xbd },
+ { 0xbe, 0xff },
 
-  { 0xd0, 0xf0 },
-  { 0xd1, 0xf1 },
-  { 0xd2, 0xf2 },
-  { 0xd3, 0xf3 },
-  { 0xd4, 0xf4 },
-  { 0xd5, 0xf5 },
-  { 0xd6, 0xf6 },
-  { 0xd8, 0xf8 },
-  { 0xd9, 0xf9 },
-  { 0xda, 0xfa },
-  { 0xdb, 0xfb },
-  { 0xdc, 0xfc },
-  { 0xdd, 0xfd },
-  { 0xde, 0xfe }
+ { 0xc0, 0xe0 },
+ { 0xc1, 0xe1 },
+ { 0xc2, 0xe2 },
+ { 0xc3, 0xe3 },
+ { 0xc4, 0xe4 },
+ { 0xc5, 0xe5 },
+ { 0xc6, 0xe6 },
+ { 0xc7, 0xe7 },
+ { 0xc8, 0xe8 },
+ { 0xc9, 0xe9 },
+ { 0xca, 0xea },
+ { 0xcb, 0xeb },
+ { 0xcc, 0xec },
+ { 0xcd, 0xed },
+ { 0xce, 0xee },
+ { 0xcf, 0xef },
+
+ { 0xd0, 0xf0 },
+ { 0xd1, 0xf1 },
+ { 0xd2, 0xf2 },
+ { 0xd3, 0xf3 },
+ { 0xd4, 0xf4 },
+ { 0xd5, 0xf5 },
+ { 0xd6, 0xf6 },
+ { 0xd8, 0xf8 },
+ { 0xd9, 0xf9 },
+ { 0xda, 0xfa },
+ { 0xdb, 0xfb },
+ { 0xdc, 0xfc },
+ { 0xdd, 0xfd },
+ { 0xde, 0xfe }
 };
 
 static int
-iso_8859_2_apply_all_case_fold(OnigCaseFoldType flag,
-			       OnigApplyAllCaseFoldFunc f, void* arg)
+apply_all_case_fold(OnigCaseFoldType flag,
+		    OnigApplyAllCaseFoldFunc f, void* arg)
 {
   return onigenc_apply_all_case_fold_with_map(
-            sizeof(CaseFoldMap)/sizeof(OnigPairCaseFoldCodes), CaseFoldMap, 1,
-            flag, f, arg);
+             sizeof(CaseFoldMap)/sizeof(OnigPairCaseFoldCodes), CaseFoldMap, 1,
+             flag, f, arg);
 }
 
 static int
-iso_8859_2_get_case_fold_codes_by_str(OnigCaseFoldType flag,
+get_case_fold_codes_by_str(OnigCaseFoldType flag,
     const OnigUChar* p, const OnigUChar* end, OnigCaseFoldCodeItem items[])
 {
   return onigenc_get_case_fold_codes_by_str_with_map(
@@ -207,29 +215,20 @@ iso_8859_2_get_case_fold_codes_by_str(OnigCaseFoldType flag,
 	     flag, p, end, items);
 }
 
-static int
-iso_8859_2_is_code_ctype(OnigCodePoint code, unsigned int ctype)
-{
-  if (code < 256)
-    return ENC_IS_ISO_8859_2_CTYPE(code, ctype);
-  else
-    return FALSE;
-}
-
-OnigEncodingType OnigEncodingISO_8859_2 = {
+OnigEncodingType OnigEncodingISO_8859_15 = {
   onigenc_single_byte_mbc_enc_len,
-  "ISO-8859-2",  /* name */
+  "ISO-8859-15",  /* name */
   1,             /* max enc length */
   1,             /* min enc length */
   onigenc_is_mbc_newline_0x0a,
   onigenc_single_byte_mbc_to_code,
   onigenc_single_byte_code_to_mbclen,
   onigenc_single_byte_code_to_mbc,
-  iso_8859_2_mbc_case_fold,
-  iso_8859_2_apply_all_case_fold,
-  iso_8859_2_get_case_fold_codes_by_str,
+  mbc_case_fold,
+  apply_all_case_fold,
+  get_case_fold_codes_by_str,
   onigenc_minimum_property_name_to_ctype,
-  iso_8859_2_is_code_ctype,
+  is_code_ctype,
   onigenc_not_support_get_ctype_code_range,
   onigenc_single_byte_left_adjust_char_head,
   onigenc_always_true_is_allowed_reverse_match
