@@ -2,7 +2,7 @@
   utf16_le.c -  Oniguruma (regular expression library)
 **********************************************************************/
 /*-
- * Copyright (c) 2002-2006  K.Kosako  <sndgk393 AT ybb DOT ne DOT jp>
+ * Copyright (c) 2002-2007  K.Kosako  <sndgk393 AT ybb DOT ne DOT jp>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -70,7 +70,11 @@ utf16le_is_mbc_newline(const UChar* p, const UChar* end)
     if (*p == 0x0a && *(p+1) == 0x00)
       return 1;
 #ifdef USE_UNICODE_ALL_LINE_TERMINATORS
-    if ((*p == 0x0d || *p == 0x85) && *(p+1) == 0x00)
+    if ((
+#ifndef USE_CRNL_AS_LINE_TERMINATOR
+	 *p == 0x0d ||
+#endif
+	 *p == 0x85) && *(p+1) == 0x00)
       return 1;
     if (*(p+1) == 0x20 && (*p == 0x29 || *p == 0x28))
       return 1;
@@ -80,7 +84,7 @@ utf16le_is_mbc_newline(const UChar* p, const UChar* end)
 }
 
 static OnigCodePoint
-utf16le_mbc_to_code(const UChar* p, const UChar* end)
+utf16le_mbc_to_code(const UChar* p, const UChar* end ARG_UNUSED)
 {
   OnigCodePoint code;
   UChar c0 = *p;
